"""Examples of styles for highlighting.
"""

from pygments.style import Style
from pygments.token import Generic


class BlackOnRed(Style):
    """An example style that highlights with black text on a red background.
    """
    default_style = ""
    styles = {
        Generic.Emph:           'bg:#f00 #fff'
    }



"""Pygments filter that uses spor to highlight ranges of code.
"""
import pathlib

from pygments.filter import Filter
from pygments.token import Generic
from spor.repository import open_repository


class SporRangeFilter(Filter):
    """A pygments filter that adds highlights to code based on spor metadata.

    This looks for spor anchors with metadata of the form `{"highlight": true}`.
    This will then mark the tokens contained in these anchors with
    `Generic.Emph` so that they are highlighted in pygments output.

    Args:
        anchors: An iterable of spor anchors to use.
    """

    def __init__(self, **options):
        super().__init__(**options)
        file_path = pathlib.Path(options['file'])
        repo = open_repository(file_path)
        self._anchors = tuple(anchor for _, anchor in repo.items()
                              if anchor.file_path == file_path.absolute())

    def _should_highlight(self, start, stop):
        "Whether the range `[start,stop]` should be highlighted."
        for anchor in self._anchors:
            if anchor.metadata.get('highlight', False):
                return (start >= anchor.context.offset and
                        stop <= (anchor.context.offset + len(anchor.context.topic)))

        return False

    def filter(self, lexer, stream):
        """Apply anchors to a stream of token to add highlighting.
        """
        start = 0
        for ttype, value in stream:
            stop = start + len(value)
            if self._should_highlight(start, stop):
                ttype = Generic.Emph
            yield ttype, value

            start = stop



import io
import os
from setuptools import setup, find_packages


def local_file(*name):
    return os.path.join(
        os.path.dirname(__file__),
        *name)


def read(*names, **kwargs):
    with io.open(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf8")
    ) as fp:
        return fp.read()


def read_version():
    """Read the `(version-string, version-info)` from
    `src/highlight_ranges/version.py`.
    """

    version_file = local_file(
        'src', 'highlight_ranges', 'version.py')
    local_vars = {}
    with open(version_file) as handle:
        exec(handle.read(), {}, local_vars)  # pylint: disable=exec-used
    return (local_vars['__version__'], local_vars['__version_info__'])


long_description = read(local_file('README.rst'), mode='rt')

setup(
    name='highlight_ranges',
    version=read_version()[0],
    packages=find_packages('src'),

    author='Sixty North AS',
    author_email='austin@sixty-north.com',
    description='Highlighting ranges in pygments',
    license='MIT',
    keywords='',
    url='http://github.com/sixty-north/highlight-ranges/',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3 :: Only',
    ],
    platforms='any',
    include_package_data=True,
    package_dir={'': 'src'},
    # package_data={'highlight_ranges': ['data/*.css']},
    install_requires=[
        'pygments',
        'spor',
    ],
    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax, for
    # example: $ pip install -e .[dev,test]
    extras_require={
        'dev': ['black', 'flake8', 'twine', 'wheel'],
        # 'doc': ['sphinx', 'cartouche'],
        'test': ['pytest'],
    },
    entry_points={
        'pygments.filters': [
            'highlight-ranges = highlight_ranges.filters:SporRangeFilter',
        ], 
        'pygments.styles': [
            'highlight-ranges-black-on-red = highlight_ranges.styles:BlackOnRed',
        ]
    },
    long_description=long_description,
)
